/**
 * @file  Channel4ModuleTable.cpp
 * @brief Implementierung der Klasse CChannel4ModuleTable.
 */
//////////////////////////////////////////////////////////////////////

//#include "StdAfx.h"
#include "Channel4ModuleTable.h"
#include <stdio.h>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
void CChannel4ModuleTable::InitClass(short channelID, short deviceID)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::InitClass [ %i, %i ]\n", __FILE__, __LINE__, channelID, deviceID);
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
CChannel4ModuleTable::~CChannel4ModuleTable() 
{
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleTable::GetErrorModule(short module, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::GetErrorModule [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, error, text);

	// Ist IV_MODULE_TABLE ?
	if (module == IV_MODULE_TABLE) {

		switch(error) {
		case 1: sprintf(text, "( %i ): Das geforderte Modul wird nicht untersttzt.", error); break;
		case 2: sprintf(text, "( %i ): Die geforderte Funktion wird nicht untersttzt.", error); break;
		case 3: sprintf(text, "( %i ): Es wurden bei der Funktion zu wenige Paramteter angegeben.", error); break;
		case 4: sprintf(text, "( %i ): Es wurden bei der Funktion zu viele Paramteter angegeben.", error); break;
		default: sprintf(text, "( %i ): Nicht eingetragener Modulfehler!", error);
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleApplication::GetErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void CChannel4ModuleTable::GetErrorFunction(short module, short function, short error, char *text)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::GetErrorFunction [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, module, function, error, text);

	if (module == IV_MODULE_TABLE) {

		switch(function) {
		case IV_FUNCTION_GET_TABLE:
		case IV_FUNCTION_GET_FIELD:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 3:sprintf(text, "( %i ): Der Subminorbestandteil (Feldname) des Tabellenfeldnamens ist unbekannt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_OPEN:
			switch(error) {
			case 1:sprintf(text, "( %i ): Der Majorbestandteil des Tabellennamens ist unbekannt.", error); break;
			case 2:sprintf(text, "( %i ): Der Minorbestandteil des Tabellennamens ist unbekannt.", error); break;
				// Standardmaessig ist die Option mit 0 angegeben. Ein gesetztes Bit, dessen Bedeutung unklar ist, ergibt diesen Fehler.
			case 3:sprintf(text, "( %i ): Unbekannte Option gesetzt.", error); break;
				// Wenn ein reservierter Parameter angegeben wurde.
			case 4:sprintf(text, "( %i ): Der angegebene Reservierte Parameter wird nicht untersttzt.", error); break;
				// Die Tabelle ist bereits mit schreib - Recht geoffnet worden.
			//case 5:sprintf(text, "( %i ): Die Tabelle kann nur lesend geoeffnet werden.", error); break;
				// Die Tabelle ist bereits mit Exclusive - Recht geoffnet worden.
			//case 6:sprintf(text, "( %i ): Die Tabelle ist z. Z. gesperrt.", error); break;
				// Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt. (Zu oft geoeffnet worden usw..)
			//case 7:sprintf(text, "( %i ): Die Nutzung der Tabelle wurde aufgrund von Resourcenmangel versagt.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_CLOSE:
			switch(error) {
				// Wenn versucht wird ein Handle zu schliessen der keiner offenen Tabelle zugehoerig ist.
			case 1:sprintf(text, "( %i ): Angegebener Handle war nicht geoffnet.", error); break;
				// Liste konnte nicht geschlossen werden.
			case 2:sprintf(text, "( %i ): Tabelle konnte nicht geschlossen werden.", error); break;
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_SET_FILTER:
		case IV_FUNCTION_TABLE_SET_FILTER_OK:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Ungueltiger Selektionsspaltenanzahl.
			case 2:sprintf(text, "( %i ): Falsche Anzahl von Selektionsspalten.", error); break;
				// Nach setzen des Filterkriteriums -> Kein Datensatz
			case 3:sprintf(text, "( %i ): Filterkriterium liefert kein Ergebnis.", error); break;
				// Spaltenname des Filter unbekannt.
			case 4:sprintf(text, "( %i ): Unbekannter Spaltenname.", error); break;
				// Unbekannter (nicht unterstuetzter) Filteroperator.
			case 5:sprintf(text, "( %i ): Unbekannter Filteroperator.", error); break;
				// Fehlender in Anfuehrungszeichenpaar des zu filterndem Spaltenwerts.
				// Keine vorhanden, unterschiedliches ein/aus '" oder fehlender ein oder Ausgang.
			case 6:sprintf(text, "( %i ): Fehler in Anfuehrungszeichenpaar des zu filterndem Spaltenwertes.", error); break;
				// Fehler im zu Filterndem Spaltenwert. z. B. Buchstaben obwohl nur Ziffern, zu lange Angabe etc.
			//case 7:sprintf(text, "( %i ): Fehler im zu filterndem Spaltenwert.", error); break;
				// Syntaxfehler im Filter. z. B. Abschluss des Filters mit Semikolon (reserviert fuer zukuenftige Erweiterung auf mehrer Spalten).
			//case 8:sprintf(text, "( %i ): Im zu setzendem Filter ist ein Syntaxfehler.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_GET_FILTER:
		case IV_FUNCTION_TABLE_CLEAR_FILTER:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Auf der angegebenen Tabelle ist kein Filter gesetzt.
			//case 2:sprintf(text, "( %i ): Kein Filter bei Tabelle angegeben.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		case IV_FUNCTION_TABLE_SKIP:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Wert fuer Origin ausserhalb des gueltigen Bereichs. (0, 1, 2)
			case 2:sprintf(text, "( %i ): Wert fuer Origin ungueltig.", error); break;
				// Sprungweite (Offset) fuehrte zu BOF (begin of file). Der Datensatzzeiger steht damit vor dem ersten Datensatz.
			case 3:sprintf(text, "( %i ): Sprung fuehrte zu BOF.", error); break;
				// Sprungweite (Offset) fuehrte zu EOF (end of file). Der Datensatzzeiger steht damit hinter dem letzten Datensatz.
			case 4:sprintf(text, "( %i ): Sprung fuehrte zu EOF.", error); break;
				// Sprung konnte aufgrund von angegebenen offset mit origin nicht durchgefuehrt werden.
				// Z. B. Filter gesetzt und Sprung auf aktuellen der nicht dem Filterkriterium entspraeche.
			case 5:sprintf(text, "( %i ): Sprung konnte nicht durchgefuehrt werden.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_SET_FIELD_VALUE:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Spaltenname wurde nicht gefunden.
			case 2:sprintf(text, "( %i ): Spaltenname unbekannt.", error); break;
				// Angegebener Wert ist zu lang
			case 3:sprintf(text, "( %i ): Zu setzender Wert ist zu lang.", error); break;
				// Formatfehler in zu setzendem Spaltenwert.
			case 4:sprintf(text, "( %i ): Formatfehler in zu setzendem Spaltenwert.", error); break;
				// Schreiben des Spaltenwerts fehlgeschlagen
			case 5:sprintf(text, "( %i ): Schreiben des Spaltenwerts fehlgeschalgen.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_TABLE_GET_FIELD_VALUE:
			switch(error) {
				// Wenn der angegebene Handle nicht existiert.
			case 1:sprintf(text, "( %i ): Angegebener Handle ist nicht geoffnet.", error); break;
				// Spaltenname wurde nicht gefunden.
			case 2:sprintf(text, "( %i ): Spaltenname unbekannt.", error); break;

			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;
		case IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD:
		case IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD:
		case IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD:
			switch(error) {
			case 0:
			default: sprintf(text, "( %i ): Nicht eingetragener Funktionsfehler!", error); break;
			}
			break;

		default: sprintf(text, "( %i ): Nicht eingetragener Fehlerblock fr Funktion!", error); break;
		}

		return;
	}

	// Ermittlung des Fehlertextes an Basisklasse abgeben.
	CChannel4ModuleApplication::GetErrorFunction(module, function, error, text);
}

//#########################################################
//####################### TABLE ###########################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::CallFunction(short module, short function, unsigned char *data, int &count)
{
	if (module == IV_MODULE_TABLE) {
		switch(function) {

		// Ermitteln der Tabellendefinition
		case IV_FUNCTION_GET_TABLE: return CallGetTable(data, count);
		// Ermitteln der Felddefinition
		case IV_FUNCTION_GET_FIELD: return CallGetField(data, count); 

		// Funktion nicht definiert
		default: return 0x0002; 
		}
	}

	return CChannel4ModuleApplication::CallFunction(module, function, data, count);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::GetTable(short deviceIDRemote, char *name, STable2 &table)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::GetTable [ %i, 0x%08X, STable2 ]\n", __FILE__, __LINE__, deviceIDRemote, name);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	count = sprintf((char*)m_data, "%s", name) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_GET_TABLE);
		return m_result;
	}

	// OK
	if (count != sizeof(STable2)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::GetTable [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Struktur uebernehmen
	table = *(STable2*)m_data;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::CallGetTable(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::CallGetTable [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	STable2 table;
	memset(&table, 0, sizeof(table));

	char *major, *minor;

	major = strtok((char*)data, ".");
	minor = strtok(NULL, ".");

	if (strcmp(major, "RECORD") == 0) {
		switch(atoi(minor)) {
		case 0:
			table.num = 0;
			strcpy(table.name, "PZE-Meldung");
			table.count = 5;
			break;
		case 1:
			table.num = 1;
			strcpy(table.name, "Status");
			table.count = 8;
			break;
		default:
			return 0x0002;
		}

		*(STable2*)data = table;
		count = sizeof(STable2);

	} else if (strstr((char*)data, "LIST") == (char*)data) {

			return 0x0002;
	} else if (strstr((char*)data, "ACCESS") == (char*)data) {

			return 0x0002;
	} else {
		
		// Majorbestandteil undefiniert.
		return 0x0001;
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::GetField(short deviceIDRemote, char *name, STable2Field &field)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::GetField [ %i, 0x%08X, STable2Field ]\n", __FILE__, __LINE__, deviceIDRemote, name);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	count = sprintf((char*)m_data, "%s", name) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_GET_FIELD);
		return m_result;
	}

	// OK
	if (count != sizeof(STable2Field)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::GetField [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Sturktur uebernehmen
	field = *(STable2Field*)m_data;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::CallGetField(unsigned char *data, int &count)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::CallGetField [ 0x%08X, %i ]\n", __FILE__, __LINE__, data, count);

	STable2Field field;
	memset(&field, 0, sizeof(field));
	char *major, *minor, *subminor;

	major = strtok((char*)data, ".");
	minor = strtok(NULL, ".");
	subminor = strtok(NULL, ".");

	if (strcmp(major, "RECORD") == 0) {

		switch(atoi(minor)) {
		case 0: {
			switch(atoi(subminor)) {
			case 0:
				field.num = 0;
				strcpy(field.name, "DU");
				field.type = TF_DT;
				field.size = 7;
				break;
			case 1:
				field.num = 1;
				strcpy(field.name, "Kennung");
				field.type = TF_A_NUM;
				field.size = 5;
				break;
			case 2:
				field.num = 2;
				strcpy(field.name, "PID");
				field.type = TF_A_NUM;
				field.size = 9;
				break;
			case 3:
				field.num = 3;
				strcpy(field.name, "IP");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 4:
				field.num = 4;
				strcpy(field.name, "Port");
				field.type = TF_A_NUM;
				field.size = 9;
				break;
			default:
				return 0x0003;
			}
			break;}
		case 1: {
			switch(atoi(subminor)) {
			case 0:
				field.num = 0;
				strcpy(field.name, "DU");
				field.type = TF_DT;
				field.size = 7;
				break;
			case 1:
				field.num = 1;
				strcpy(field.name, "Feld1");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 2:
				field.num = 2;
				strcpy(field.name, "Feld2");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 3:
				field.num = 3;
				strcpy(field.name, "Feld3");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 4:
				field.num = 4;
				strcpy(field.name, "Feld4");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 5:
				field.num = 5;
				strcpy(field.name, "Feld5");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 6:
				field.num = 6;
				strcpy(field.name, "Feld6");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			case 7:
				field.num = 7;
				strcpy(field.name, "Feld7");
				field.type = TF_A_NUM;
				field.size = 21;
				break;
			default:
				return 0x0003;
			}
			break;}
		default:
			return 0x0002;
		}

		*(STable2Field*)data = field;
		count = sizeof(STable2Field);

	} else if (strcmp(major, "LIST") == 0) {

			return 0x0002;
	} else if (strcmp((char*)data, "ACCESS") == 0) {

			return 0x0002;
	} else {
		
		// Majorbestandteil undefiniert.
		return 0x0001;
	}

	return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableOpen(short deviceIDRemote, char *name, int options, int &handle, char *reserved)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableOpen [ %i, 0x%08X, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, options, handle, reserved);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	count = sprintf((char*)m_data, "%s", name) + 1;

	*(int*)&m_data[count] = options;
	count += sizeof(int);

	count += sprintf((char*)&m_data[count], "%s", reserved) + 1;


	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_OPEN);
		return m_result;
	}

	// OK
	// Ein leerbyte fuer reservierten String in Antwort
	if (count != (sizeof(int) + sizeof(unsigned char))) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableOpen [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Handle uebernehmen
	handle = *(int*)m_data;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableClose(short deviceIDRemote, int handle)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableClose [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, handle);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLOSE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableClose [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetFilter(short deviceIDRemote, int handle, char *filter)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableSetFilter [ %i, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, filter);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);
	count += sprintf((char*)&m_data[count], "%s", filter) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableSetFilter[ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}
/*
	short module, function, error = IV_ERROR_SUCCESS;
	// nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
	// dieses musz hier seperat abgegriffen werden.
	if ((m_result = Pending(deviceIDRemote, module, function, error, m_data, count)) != IV_ERROR_SUCCESS) {

		Trace("%s:(%i):ERROR: BeginGetSegment.Pending [ Quittierungsprfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, m_result);
		return m_result;
	}

	if ((module == IV_MODULE_TABLE) && (function == IV_FUNCTION_TABLE_SET_FILTER_OK)) {

		Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableSetFilter.Pending [ Antwort OK ]\n", __FILE__, __LINE__);
	}

	// evntl. aktiven Fehler auswerten
	if (error != IV_ERROR_SUCCESS) {

		m_result = error;
		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK);
		return m_result;
	}
*/
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FILTER_OK);
		return m_result;
	}

	// Keine Aktive - Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableSetFilter.Pending [ Aktive - Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetFilter(short deviceIDRemote, int handle, char *filter, short &len)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGetFilter [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, filter, len);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter uebernehmen
	*(int*)m_data = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FILTER);
		return m_result;
	}

	// OK
	if (count > 80) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetFilter [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, count);
	}

	// Wert uebernehmen
	strcpy(filter, (char*)m_data);
	len = strlen(filter);

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableClearFilter(short deviceIDRemote, int handle)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableClearFilter [ %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, handle);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_CLEAR_FILTER);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableClearFilter [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSkip(short deviceIDRemote, int handle, int offset, short origin)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableSkip [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, offset, origin);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);
	*(int*)&m_data[count] = offset;
	count += sizeof(int);
	*(short*)&m_data[count] = origin;
	count += sizeof(short);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SKIP);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableSkip [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}


// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetField(short deviceIDRemote, int handle, char *name, char *value)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableSetField [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, deviceIDRemote, name, value);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);
	count += sprintf((char*)&m_data[count], "%s", name) + 1;
	count += sprintf((char*)&m_data[count], "%s", value) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_FIELD_VALUE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableSetField [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetField(short deviceIDRemote, int handle, char *name, char *value, short &len)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGetField [ %i, 0x%08X, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, name, value, len);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)m_data = handle;
	count = sizeof(int);
	count += sprintf((char*)&m_data[count], "%s", name) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_FIELD_VALUE);
		return m_result;
	}

	// OK
	if (count > 80) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetField [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Wert uebernehmen
	strcpy(value, (char*)m_data);
	len = strlen(value);

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableSetRecord(short deviceIDRemote, int handle, unsigned char *data, short len)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableSetRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, handle, data, len);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)&m_data[count] = handle;
	count = sizeof(int);
	memcpy(&m_data[count], data, len);
	count += len;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_SET_RECORD);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableSetRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecord(short deviceIDRemote, int handle, unsigned char *data, short &len)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGetRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, handle, data, len);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)&m_data[count] = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD);
		return m_result;
	}

	// OK
	if (count > 256) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	if (count > len) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetRecord [ Puffer fr Antwortdaten zu klein gewhlt. ]\n", __FILE__, __LINE__, count);
		// Wert uebernehmen
		memcpy(data, (char*)m_data, len);
	} else {

		// Wert uebernehmen
		memcpy(data, (char*)m_data, count);
		len = count;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecordCount(short deviceIDRemote, int handle, int &recordCount)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGetRecordCount [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, recordCount);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)&m_data[count] = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_COUNT);
		return m_result;
	}

	// OK
	if (count > sizeof(int)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetRecordCount [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Anzahl uebernehmen
	recordCount = *(int*)m_data;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGetRecordNum(short deviceIDRemote, int handle, int &recordNum)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGetRecordNum [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, recordNum);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Name als Parameter eintragen
	*(int*)&m_data[count] = handle;
	count = sizeof(int);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GET_RECORD_NUM);
		return m_result;
	}

	// OK
	if (count > sizeof(int)) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGetRecordCount [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Anzahl uebernehmen
	recordNum = *(int*)m_data;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableAppendRecord(short deviceIDRemote, unsigned char type, unsigned char *data, int size)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::FingerprintTableAppendRecord [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type, data, size);

	int count = 0, i;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	m_data[count] = type;
	count = sizeof(unsigned char);

	for (i=0; i < size; i++) {
	
		m_data[count + i] = data[i];
	}
	count += size;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD);
		return m_result;
	}

	// OK
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableAppendRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	// Aktive Antwort abwarten
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_APPEND_RECORD_OK);
		return m_result;
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableGetRecord(short deviceIDRemote, unsigned char &type, unsigned long pid, unsigned char fid, unsigned char *data, int &size)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::FingerprintTableGetRecord [ %i, %i, %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, deviceIDRemote, type, pid, fid, data, size);

	int count = 0, i;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	m_data[count] = type;
	count = sizeof(unsigned char);
	*(unsigned long*)&m_data[count] = pid;
	count += sizeof(unsigned long);
	m_data[count] = fid;
	count += sizeof(unsigned char);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_GET_RECORD);
		return m_result;
	}

	// Templatetype uebernehmen
	type = m_data[0];
	// Templatedaten uebernehmen
	size = count - 1;
	for (i=1; i < count; i++) {
	
		data[i - 1] = m_data[i];
	}
	count += size;

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::FingerprintTableDeleteRecord(short deviceIDRemote, unsigned long pid, unsigned char fid)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::FingerprintTableDeleteRecord [ %i, %i, %i ]\n", __FILE__, __LINE__, deviceIDRemote, pid, fid);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);


	// Parameter eintragen
	*(unsigned long*)&m_data[count] = pid;
	count = sizeof(unsigned long);
	m_data[count] = fid;
	count += sizeof(unsigned char);

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(deviceIDRemote, IV_MODULE_TABLE, IV_FUNCTION_FINGERPRINT_TABLE_DELETE_RECORD);
		return m_result;
	}

	// OK
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::FingerprintTableDeleteRecord [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	}

	return m_result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceIDRemote Anzusprechende Geraetenummer.
 */
int CChannel4ModuleTable::TableGenerate(short deviceIDRemote, char *name, char *reserved)
{
	Trace("%s:(%i):INFO(3): CChannel4ModuleTable::TableGenerate [ %i, %s, %s ]\n", __FILE__, __LINE__, deviceIDRemote, name, reserved);

	int count = 0;

	// Umgebung zuruecksetzen
	Reset();

	// Remoteadresse setzen.
	SetDeviceIDRemote(deviceIDRemote);

	// Alle Zusatzparameter uebernehmen
	count += sprintf((char*)&m_data[count], "%s", name) + 1;
	count += sprintf((char*)&m_data[count], "%s", reserved) + 1;

	if ((m_result = Execute(IV_CONTEXT_COMMAND, IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE, m_data, count)) != IV_ERROR_SUCCESS) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUNCTION_TABLE_GENERATE);
		return m_result;
	}

	// Keine Antwortdaten erwartet
	if (count) {

		Trace("%s:(%i):WARNING(1): CChannel4ModuleTable::TableGenerate [ Antwort enhaelt %i Byte an unntze Daten. ]\n", __FILE__, __LINE__, count);
	} else {

		count = 0;
	}

	// Aktive Antwort abwarten
	if ((m_result = Execute(IV_CONTEXT_PENDING, IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK, m_data, count)) != IV_ERROR_SUCCESS ) {

		// Fehler ausgeben
		TraceError(GetDeviceIDRemote(), IV_MODULE_TABLE, IV_FUCNTION_TABLE_GENERATE_OK);
		return m_result;
	}

	return m_result;
}

